##  Copyright 2005, 2005 Kieran Martin

##    This program is free software; you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation; either version 2 of the License, or
##    (at your option) any later version.
##
##    This program is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with this program; if not, write to the Free Software
##    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##
    
import pickle
import re
import wx
import win32api
import string
import sys
import os
import xmlrpclib
import socket
import wx.lib.layoutf       as layoutf
import wx.lib.imagebrowser  as ib
import wx.lib.scrolledpanel as scrolled
from wxPython.lib.evtmgr import eventManager
import wx.lib.colourdb
#from cStringIO import StringIO

def cleanup(id):
    """Removes spaces and fullstops from potential id Names, also makes it start with a letter."""
    p = re.compile('^[A-Za-z]')
    if not p.match(id):
        id = 'a' + id
    id = id.replace('.', '_')
    id = id.replace('-', '_')
    id = id.replace("'", '')
    return id.replace(' ', '_')

def esc(id):
    return id.replace('"',"'")
    
def opj(path):
    """Convert paths to the platform-specific separator"""
    return apply(os.path.join, tuple(path.split('/')))

def prepend(list, item):
    """Adds an item to the start of a list"""
    list.reverse
    list.append( item )
    list.reverse
    return list

class MySplashScreen(wx.SplashScreen):
    def __init__(self):
        bmp = wx.Image(opj("bitmaps/splash.gif")).ConvertToBitmap()
        wx.SplashScreen.__init__(self, bmp,
                                 wx.SPLASH_CENTRE_ON_SCREEN | wx.SPLASH_NO_TIMEOUT,
                                 30000, None, -1)

#---------------------------------------------------------------------------
class MyLog(wx.PyLog):
    def __init__(self, app, textCtrl, logTime=0):
        wx.PyLog.__init__(self)
        self.log = textCtrl
        self.logTime = logTime

    def DoLogString(self, message, timeStamp):
        if self.logTime:
            message = time.strftime("%X", time.localtime(timeStamp)) + \
                      ": " + message
        if self.log:
            self.log.AppendText(message + '\n')
#---------------------------------------------------------------------------

class PhotoPanel(wx.Panel):
    """Contains one Photo (and associated fields on the screen"""
    def Delete(self):
        self.app.tree.deletePhoto( self.imageId )
        
    def stripe(self,app):
        self.SetBackgroundColour(app.c3)
        if self.bgcolor:
            self.SetBackgroundColour(app.c2)
        self.Refresh()

    def __init__(self, parent, app, imageId, image, bgcolor ):
        """imageId - the zope name of the Photo
           image   - a hashlist of all Photo properties and data
           bgcolor   - a binary - flipflop to decide background color"""
    
        wx.Panel.__init__(self,parent,-1)
        self.app = app
        self.parent = parent
        self.imageId = imageId
        self.bgcolor = bgcolor
        
        self.stripe(app)

        if image.has_key('data'):
            app.tree.cacheImage( imageId, image['data'])
        
        props = image['Properties']

        if image['filename'] == 'empty':
            jpg = wx.Image(opj("bitmaps/noimage.jpg"), wx.BITMAP_TYPE_JPEG).ConvertToBitmap()            
        else:
            jpg = app.tree.readCache( imageId )

        gbs = wx.GridBagSizer(5,5)
        gbs.SetVGap(10)
        
        fields = app.config['fields']
        for field in fields:

            if not props.has_key(field):
                props[field] = ''

        self.pic            = wx.StaticBitmap(self, -1, jpg)
        self.pic.SetCursor( wx.StockCursor( wx.CURSOR_HAND ) )
        self.pic.dataobject = wx.TextDataObject( 'P' + self.imageId )                                

        self.tname   = wx.StaticText( self, -1, imageId )
        self.ttitle  = wx.TextCtrl(   self, -1, image['Title']  )
        self.t1 = wx.TextCtrl(   self, -1, props[fields[0]] )
        self.t2  = wx.TextCtrl(   self, -1, props[fields[1]] )        
        self.t3 = wx.TextCtrl(   self, -1, props[fields[2]] )        
        self.tdesc   = wx.TextCtrl(   self, -1, image['Description'], size=wx.Size(500,65), style=wx.TE_MULTILINE  )
       
        fonth = wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD)
        fontm = wx.Font(12, wx.SWISS, wx.NORMAL, wx.NORMAL)
        self.tname.SetFont(fonth)
        b = 10

        gbs.Add(wx.StaticText(self,-1,'Title'),      (1,2), flag=wx.ALIGN_RIGHT | wx.ALL,  border=b)
        gbs.Add(wx.StaticText(self,-1,'Description'),(3,2), flag=wx.ALIGN_RIGHT | wx.ALL,  border=b)
        gbs.Add(wx.StaticText(self,-1,fields[0]),     (1,4), flag=wx.ALIGN_RIGHT | wx.ALL,  border=b)
        gbs.Add(wx.StaticText(self,-1,fields[1]),      (2,2), flag=wx.ALIGN_RIGHT | wx.ALL,  border=b)
        gbs.Add(wx.StaticText(self,-1,fields[2]),     (2,4), flag=wx.ALIGN_RIGHT | wx.ALL,  border=b)

        gbs.Add(self.tname,                 (4,1),           flag=wx.ALIGN_TOP , border=b)
        gbs.Add(self.pic,                   (1,1),(3,1))
        gbs.Add(self.ttitle,                (1,3),           border=b)
        gbs.Add(self.t1,               (1,5),           border=b)
        gbs.Add(self.t2,                (2,3),           border=b)
        gbs.Add(self.t3,               (2,5),           border=b)        
        gbs.Add(self.tdesc,(3,3),wx.GBSpan  (2,3),           border=b)

        eventManager.Register( self.OnPicMouseOver,   wx.EVT_ENTER_WINDOW, self.pic )
        eventManager.Register( self.OnPicMouseOut,   wx.EVT_LEAVE_WINDOW, self.pic )

        eventManager.Register( self.OnLeftDown,   wx.EVT_LEFT_DOWN, self.pic )
        eventManager.Register( self.changeTitle,  wx.EVT_TEXT,      self.ttitle )
        eventManager.Register( self.changeName,   wx.EVT_TEXT,      self.tname )
        eventManager.Register( self.change4,   wx.EVT_TEXT,      self.tdesc )
        eventManager.Register( self.change1, wx.EVT_TEXT,      self.t1 )
        eventManager.Register( self.change2,  wx.EVT_TEXT,      self.t2 )
        eventManager.Register( self.change3, wx.EVT_TEXT,      self.t3 )

        self.SetSizerAndFit(gbs)      

    def Deselect(self):
        """ called by the parent when another child has been selected """
        self.stripe( self.app )
        
    def OnPicMouseOver( self, event ):
        self.tname.SetForegroundColour( wx.Colour( 0, 0, 250 ) )
        self.tname.Refresh()

    def OnPicMouseOut( self, event ):
        self.tname.SetForegroundColour( wx.Colour( 0, 0, 0 ) )
        self.tname.Refresh()

    def OnLeftDown(self,event):
        """Sets up data should enabling a drag+drop move using the
           thumbnail image"""

        ds = wx.DropSource(self.pic)
        ds.SetData( self.pic.dataobject )
        self.SetBackgroundColour( self.app.csel )
        self.Refresh()
        self.parent.Select(self)
        try:
            result = ds.DoDragDrop()
        except:
            print 'Drag and drop operation failed.'
            
    def changeName(self,event):
        app.tree.makedirty('Name', self.imageId)
    def changeTitle(self,event):
        app.tree.makedirty('Title', self.imageId)
    def change4(self,event):
        app.tree.makedirty('Description', self.imageId)
    def change1(self,event):
        app.tree.makedirty(app.config['fields'][0], self.imageId)
    def change2(self,event):
        app.tree.makedirty(app.config['fields'][1], self.imageId)
    def change3(self,event):
        app.tree.makedirty(app.config['fields'][2], self.imageId)
        
class FolderPanel(wx.Panel):
    """Appears at the top of the main panel. Allows folder details to be maintained."""
    def Delete(self):
        self.app.tree.deleteFolder(  )
    
    def __init__(self, parent, app ):
        """imageId - the zope name of the Photo
           image   - a hashlist of all Photo properties and data
           bgcolor   - a binary - flipflop to decide background color"""
    
        wx.Panel.__init__(self,parent,-1)
        self.app = app
        self.parent = parent
        self.SetBackgroundColour(app.c1)
        bmp = wx.Image(opj("bitmaps/photoalbum.gif")).ConvertToBitmap()               

        gbs = wx.GridBagSizer(5,5)
        gbs.SetVGap(10)
        
        self.id = app.tree.getCurrentPath()[-1]
        self.folder = app.tree.getCurrentFolder()
        
        self.pic            = wx.StaticBitmap(self, -1, bmp)
        self.pic.SetCursor( wx.StockCursor( wx.CURSOR_HAND ) )
        self.pic.dataobject = wx.TextDataObject( 'F' )              

        self.tname   = wx.StaticText( self, -1, ' Folder: ' + self.id )
        self.ttitle  = wx.TextCtrl(   self, -1, self.folder['Title']      )
        self.tdesc   = wx.TextCtrl(   self, -1, self.folder['Description'], size=wx.Size(500,65) )
       
        fonth = wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD)
##        fonth.SetWeight( wx.BOLD )
        fontm = wx.Font(12, wx.SWISS, wx.NORMAL, wx.NORMAL)
        self.tname.SetFont(fonth)
        b = 10

        gbs.Add(wx.StaticText(self,-1,'Title'),      (1,2), flag=wx.ALIGN_RIGHT | wx.ALL,  border=b)
        gbs.Add(wx.StaticText(self,-1,'Description'),(3,2), flag=wx.ALIGN_RIGHT | wx.ALL,  border=b)

        gbs.Add(self.pic,                   (1,1),(4,1))
        gbs.Add(self.ttitle,(1,3),wx.GBSpan  (1,3),           border=b)
        gbs.Add(self.tdesc,(2,3) ,wx.GBSpan  (3,3),           border=b)

        eventManager.Register( self.OnLeftDown,   wx.EVT_LEFT_DOWN, self.pic )
        eventManager.Register( self.changeTitle,  wx.EVT_TEXT,      self.ttitle )
        eventManager.Register( self.change4,   wx.EVT_TEXT,      self.tdesc )

        self.SetSizerAndFit(gbs)      

    def Deselect(self):
        """ called by the parent when another child has been selected """
        self.SetBackgroundColour( self.app.c1 )
        self.Refresh()
        
    def OnLeftDown(self,event):
        """Sets up data should enabling a drag+drop move using the
           thumbnail image"""

        ds = wx.DropSource(self.pic)
        ds.SetData( self.pic.dataobject )
        self.SetBackgroundColour( self.app.csel )
        self.Refresh()
        self.parent.Select(self)        
        try:
            result = ds.DoDragDrop()
        except:
            print 'Drag and drop operation failed.'
            
    def changeTitle(self,event):
        app.tree.makedirty('Title')
    def change4(self,event):
        app.tree.makedirty('Description')


class ThumbNail( wx.Panel ):

    def __init__( self, parent, app, dir , file ):
        wx.Panel.__init__(self,parent,-1, size=(140,140),style=wx.STATIC_BORDER)
        self.parent = parent
        self.path = dir + '\\' + file
        print self.path
        cachename = self.path.replace('//','~')
        cachename = cachename.replace('\\','~')        
        cachename = cachename.replace('c:~','')
        cachename = cachename.replace('C:~','')        
##        print 'cachename ' + cachename
        pic = ''
        if not os.access( app.PCcachedir + cachename, os.R_OK ):
            pic = wx.Image( self.path, wx.BITMAP_TYPE_JPEG )
            if pic.Ok():
                if pic.GetWidth() > pic.GetHeight():
                    pic.Rescale( 120, 90 )
                else:
                    pic.Rescale( 90, 120 )
            else:
                pic = wx.Image(opj("bitmaps/noimage.jpg"), wx.BITMAP_TYPE_JPEG)
                
            pic.SaveFile(app.PCcachedir + cachename, wx.BITMAP_TYPE_JPEG  )
        else:
            print 'loading file ' + app.PCcachedir + cachename
            pic = wx.Image( app.PCcachedir + cachename, wx.BITMAP_TYPE_JPEG   )
            
        pic = wx.StaticBitmap(self, -1, pic.ConvertToBitmap())
        self.pic = pic
        self.pic.SetCursor( wx.StockCursor( wx.CURSOR_HAND ) )
        pic.dataobject = wx.TextDataObject( 'I' + self.path )                                
        eventManager.Register( self.OnLeftDown,   wx.EVT_LEFT_DOWN, self.pic )

        bx = wx.BoxSizer( wx.VERTICAL )

        bx.Add( pic )
        bx.Add( wx.StaticText( self, -1 , file ) )

        self.SetSizerAndFit( bx )
##        print self.GetSizeTuple()


    def OnLeftDown(self,event):
        """Sets up data should enabling a drag+drop move using the
           thumbnail image"""

        ds = wx.DropSource(self.pic)
        ds.SetData( self.pic.dataobject )
        try:
            result = ds.DoDragDrop()
        except:
            print 'Drag and drop operation failed.'
        
class FolderNail( wx.Panel ):
    def __init__( self, parent, dir , file ):
        wx.Panel.__init__(self,parent,-1, size=(140,140),style=wx.NO_BORDER)
        self.parent = parent
        self.path = dir + '\\' + file
        ##        self.SetBackgroundColour('blue')
        bx = wx.BoxSizer( wx.VERTICAL )
        pic = wx.StaticBitmap(self, -1, wx.Image(opj( 'bitmaps//folder.gif' ), wx.BITMAP_TYPE_GIF).ConvertToBitmap())

        bx.Add( pic )
        self.ttitle  = wx.StaticText(self, -1, file)
        bx.Add( self.ttitle, 0, wx.EXPAND )
        self.SetSizerAndFit( bx )
        eventManager.Register( self.open, wx.EVT_LEFT_DCLICK, pic )
##        print self.GetSizeTuple()        
    def open(self,event):
        print 'opening folder: ' + self.path
        self.parent.loadpanel(self.path)
                
class BackNail( wx.Panel ):
    def __init__( self, parent, dir ):
        wx.Panel.__init__(self,parent,-1, size=(140,140))
        self.parent = parent

        p = re.compile('\\$')   #  drop the last \ in the path
        dir = p.sub('',dir)
        self.path,crap = os.path.split(dir)

        bx = wx.BoxSizer( wx.VERTICAL )
        pic = wx.StaticBitmap(self, -1, wx.Image(opj( 'bitmaps//uparrow.bmp' ), wx.BITMAP_TYPE_BMP).ConvertToBitmap())

        bx.Add( pic )
##        self.ttitle  = wx.StaticText(self, -1, self.path)
##        bx.Add( self.ttitle, 0, wx.EXPAND )
        self.SetSizerAndFit( bx )
        eventManager.Register( self.open, wx.EVT_LEFT_DCLICK, pic )
##        print self.GetSizeTuple()
    def open(self,event):
        print 'backnailing to: ' + self.path
        self.parent.loadpanel(self.path)


class CameraPanel(scrolled.ScrolledPanel):
    """This is the right hand scrolling panel - it contains
       the PhotoPanels"""
    
    def __init__( self, parent, app ):
        self.parent = parent
        self.app = app
        scrolled.ScrolledPanel.__init__(self, parent, -1 )
        self.SetBackgroundColour('black')
        self.loadpanel( app.config['cameradir'] )
        
    def loadpanel( self, dir ):

        [width, height] = self.parent.GetSizeTuple()
        cols = width / 140
        if width < 100:
            cols = 4
        self.DestroyChildren()        
        gs = wx.FlexGridSizer( 0, cols , 5 ,5)
        gs.Add( BackNail( self, dir ) )

        max = 5
        dlg = wx.ProgressDialog("Making Thumbnails",
                               "Please Wait",
                               maximum = max,
                               parent=self.parent,
                               style = wx.PD_CAN_ABORT | wx.PD_APP_MODAL)

        keepGoing = True
        count = 0
        print 'dir is ' + dir
        try:
            files = os.listdir( dir )
        except:
            dlg.Destroy()
            alert = wx.MessageDialog( self.app.frame, "You don't have authority to view the contents of this folder.", "Restricted Folder.",style=wx.OK)
            alert.ShowModal()
            return  # windows didn't let us near this dir

        print str(len(files)) + ' in ' + dir
        tithe=5
        if len( files ) > 5:
            tithe = len(files) / 5
        blip = 0
        for file in os.listdir( dir ):
##            print 'file found ' + file
            blip = blip + 1
            if blip > tithe:
                blip = 0
                count = count + 1
                dlg.Update( count )

            if os.path.isdir( dir + '\\' + file ):
                gs.Add( FolderNail( self, dir, file ))

            if os.path.isfile( dir + '\\' +  file ):
                if ( string.find( file, '.jpeg' ) > -1 ) or \
                   ( string.find( file, '.jpg' ) > -1 ) or \
                   ( string.find( file, '.JPG' ) > -1 ) or \
                   ( string.find( file, '.JPEG' ) > -1 ) :
                    gs.Add( ThumbNail( self, self.app, dir, file ) )

        dlg.Destroy()
        self.SetSizer( gs )
        self.SetAutoLayout( 1 )
        self.SetupScrolling()
        self.Layout()
        
class TestPanel( scrolled.ScrolledPanel ):
    """This is the right hand scrolling panel - it contains
       the PhotoPanels"""
    
    def __init__( self, parent, app ):

        scrolled.ScrolledPanel.__init__(self, parent, -1 )
        self.SetBackgroundColour( app.c1 )
        self.selected = []

    def loadPanel( self, tree ):
        """Using the Photos in the current node, builds the scrolled
           display of Photos in the main panel"""

        if len(tree.getCurrentPath()) == 0:
            return
        
        bgcolor = 0
        self.DestroyChildren()
        bx = wx.BoxSizer(wx.VERTICAL)
        self.bx = bx

        folderListing = app.tree.getCurrentNode()
        bx.Add( FolderPanel( self, app ), flag=wx.EXPAND )

        for imageId in folderListing.keys():
            image = folderListing[imageId]
            if image['Type'] =='Photo':
                bgcolor = ~bgcolor
                bx.Add( PhotoPanel( self, app, imageId, image, bgcolor ), flag=wx.EXPAND )

        self.SetSizer( bx )
        self.SetAutoLayout( 1 )
        self.SetupScrolling()
        self.Layout()

    def Delete( self ):
        if len( self.selected ) == 0:
            return
        for item in self.selected:
            item.Delete()
        self.selected = []
        return self.loadPanel( app.tree )

        
    def Select( self, item ):
        """ called by one of its children - deselect the current object and select this one """
        if not len( self.selected ) == 0:
            for i in self.selected:
                try:
                    i.Deselect()
                except wx._core.PyDeadObjectError:
                    None
            self.selected = []
        self.selected.append(item)
            

class MyTextDropTarget( wx.TextDropTarget ):
    """ The target of drag/drop moves of folders and photos
        into the treeCtrl """
    
    def __init__(self, parent, app):
        wx.TextDropTarget.__init__(self)
        self.app = app
        
    def OnDropText(self, x, y, fromstr):
        """Uses the coordinates to get the treeCtrl node - this is the target.
           The fromstr contains a path or id name of the thing being dragged."""

        tree       = self.app.tree
        targetItem = tree.HitTest(wx.Point(x,y))

        if   fromstr[0] == 'P':
            if not tree.movePhoto(  targetItem[0], fromstr[1:] ):
                return 0

        elif fromstr[0] == 'F':
            if not tree.moveFolder( targetItem[0] ):
                return 0

        elif fromstr[0] == 'I':
            if not tree.addPhoto(  targetItem[0], fromstr[1:] ):
                return 0

        return self.app.frame.weblistPanel.loadPanel( tree )

    def OnDragOver(self, x, y, d):
        """Need this to enable the drag/drop functions. Not sure why! """
        return wx.DragCopy

class MyTree( wx.TreeCtrl ):
    """This class unifies the 4 main data elements in this program:
        1.  The treecontrol - a wx.Windows tree (accessed via GetParent etc...)
        2.  images - the local tree             (accessed via path[])
        3.  xmlrpc - the on-site repository -   (accessed via path[])
        4.  The local image cache.              (accessed via path)  """

    def __getFolder(self, inpath):
        """Given the path as an array returns the folder"""
        path = []
        path.extend(inpath)
        path.reverse()
        here = self.images

        while (len(path) > 0):
            y = path.pop()
            try:
                here = here['Node']
            except:
                None

            here = here[y]  

        return here
    
    def __getNode(self, inpath):
        """Given the path as an array returns the node"""
        path = []
        path.extend(inpath)
        path.reverse()
        here = self.images

        while (len(path) > 0):
            y = path.pop()
            try:
                here = here['Node']
            except:
                None

            here = here[y]  

        try:
            images = here['Node']
        except KeyError:
            images=here

        return images
    
    def __getPath(self, node):
        """Given a node name returns its path as an array and its id"""
        def walkPath( node):
            """ recurses back thru the tree making a path in array form..  .."""
            path.append( self.GetItemText( node ) )

            if self.GetItemParent( node ):
                walkPath( self.GetItemParent( node ) )
            return
        path = []

        walkPath( node)
        path.pop()
        path.reverse()

        return path
    

    def loadtree(self, treeNode, zopeNode, loadcount):
        """ Recurses through zope tree loading up the treeCtrl
            Returns the number of images not in the local cache. """
        for x in zopeNode.keys():
            if zopeNode[x]['Type'] == 'Photo Album':
                child = self.AppendItem(treeNode, x)
                self.SetItemImage(child, self.fldridx, wx.TreeItemIcon_Normal)
                self.SetItemImage(child, self.fldropenidx, wx.TreeItemIcon_Expanded)
                self.SetItemText(child, x)
                loadcount = self.loadtree(child, zopeNode[x]['Node'],loadcount)
            else:
                node = zopeNode[x]
                if node['filename'] == 'found!!':
                    loadcount = loadcount + 1
#                    else:
#                        print node['filename']
        self.SetImageList(self.app.il)                    
        return loadcount        
    
    def __perform(self,action):
        """performs an xmlrpc action then logs it"""
        action = action.replace('..','.')
        wx.LogMessage('#### ACTION ###')
        wx.LogMessage(action)
        try:
            eval("%s" % (action))
        except xmlrpclib.ProtocolError:
            return 1
        except socket.error:
            alert = wx.MessageDialog( self.app.frame, "This operation was not completed. Reconnect with the internet and try again.", "Lost Connection.",style=wx.OK)
            alert.ShowModal()
            return 1
        except xmlrpclib.Fault, zoperror:
            if (string.find(str(zoperror), '- it is already in use')):
                alert = wx.MessageDialog( self.app.frame, "A photo with this name already exists in this folder. No action taken.",style=wx.OK)
                alert.ShowModal()
                return 0
            
            alert = wx.MessageDialog( self.app.frame, "You may be out of sync with the website. If problems persist. restart this program.", "Operation Failed on Website.",style=wx.OK)
            alert.ShowModal()
            print wx.LogMessage(action + "operation failed.....")
            alert = wx.MessageDialog( self.app.frame, str(zoperror), "Website detected error",style=wx.OK)
            alert.ShowModal()

            return 0
        return 1

    def __expandtree(self,branch):
        """Recurses the treeCtrl, opening each parent node."""
        self.Expand(branch)
        if self.ItemHasChildren(branch):
            (z,c) = self.GetFirstChild(branch)
            while z:
                self.__expandtree(z)
                (z,c) = self.GetNextChild(branch,c)
        z = self.GetNextSibling(branch)
        while z:
            self.__expandtree(z)
            z = self.GetNextSibling(z)
    
    def getCurrentPath(self):
        return self.__getPath( self.GetSelection() )
    
    def getCurrentNode(self):
        return self.__getNode( self.getCurrentPath() )
    
    def getCurrentFolder(self):
        return self.__getFolder( self.getCurrentPath() )
    
    def __init__(self, app):
        """Create the treectrl, log on to the site and
        get the data. """
        wx.TreeCtrl.__init__(self, app.frame.leftPanel, -1,wx.DefaultPosition, (20000,10000), wx.TR_HAS_BUTTONS | wx.TR_TWIST_BUTTONS | wx.TR_FULL_ROW_HIGHLIGHT | wx.NO_BORDER)

        self.movedatalater = []
        self.selChange = 1
        self.SelectRoot = False
        
        self.dirty = []
        self.deleting = False
        self.app = app
        self.SetBackgroundColour( app.c1 )
        self.Bind(wx.EVT_TREE_SEL_CHANGED,  self.OnSelChanged )
        self.Bind(wx.EVT_TREE_SEL_CHANGING, self.OnSelChanging )

        self.fldridx     = app.il.Add(wx.ArtProvider_GetBitmap(wx.ART_FOLDER,      wx.ART_OTHER, app.isz))
        self.fldropenidx = app.il.Add(wx.ArtProvider_GetBitmap(wx.ART_FILE_OPEN,   wx.ART_OTHER, app.isz))

        self.root = self.AddRoot("My Gallery")
        self.SetItemImage(self.root, self.fldridx, wx.TreeItemIcon_Normal)
        self.SetItemImage(self.root, self.fldropenidx, wx.TreeItemIcon_Expanded)
        self.SetPyData(self.root, None)

        self.xmlrpc = self.__login()
##        if not self.xmlrpc:
##            return
        
        wx.WindowDisabler()
        wx.BusyInfo("Please wait while your gallery data is copied from the web.")
        self.images = self.xmlrpc.imagetree(string.join(app.pix,','))

        loadcount = self.loadtree(self.root,self.images,0)
        wx.LogMessage(str(loadcount) + ' images preloaded')

        droptarget = MyTextDropTarget(self,app)

        self.SetDropTarget(droptarget)

        self.__expandtree(self.root)

        
    def makedirty(self, field, name=0 ):
        path = string.join( self.getCurrentPath(), '~' )
        change = {'field':field, 'item':name, 'path':path}
        if change in self.dirty:
            None
        else:
            self.dirty.append(change)
        return 1
            
    def cacheImage( self, name, image ):
        path = string.join( self.getCurrentPath(), '~' )
        filename = self.app.cachedir + path + '~' + name + '.jpg' 
        f = open(filename, 'wb')
        f.write(image.data)
        f.close()
        return 1

 
    def readCache( self, name ):
        path = string.join( self.getCurrentPath(), '~' )
        print 'readcache path: ' + path + name
        return wx.Image(opj( self.app.cachedir + path + '~' + name + '.jpg' ), wx.BITMAP_TYPE_JPEG).ConvertToBitmap()

##     INTERNAL METHODS        

    def __login(self):
        """Collect the userid and password
        then connect to the site, returning an
        XMLRPC Server object    """
        config = self.app.config
        logged_in = 0
        login_string = 'Enter your password'

        while not logged_in:
            logged_in = 1
            passw = wx.GetPasswordFromUser(login_string,'Connecting ' + config['userid'] + ' to ' + config['url'])

            if len(passw) == 0:
                return False
                
            v = xmlrpclib.ServerProxy(config['url'], password=passw, userid=config['userid'])
            try:
                v.access_log.setTitle('access_log')
            except xmlrpclib.ProtocolError:
                logged_in = 0
                login_string = 'Wrong password!  Try again:'
        return v
    
##  HANDLERS
            

    def OnSelChanging(self,event):
        """don't allow selection of the root folder"""
        if event.GetItem().m_pItem == self.GetRootItem().m_pItem:
            if not self.SelectRoot:
                event.Veto()

        if self.selChange == 0:
            event.Veto()
        self.selChange = 1

    def OnSelChanged(self, event):
        """User selected another folder. Copy the current folder
        details to the tree, then reload the screen"""

        if self.app.frame.notebookPanel.GetSelection():
            self.app.frame.notebookPanel.AdvanceSelection(False)

        if not self.deleting:
            try:
                self.copyScreenToTable( self.__getPath( event.GetOldItem() ) )
            except wx._core.PyAssertionError:
                None
##                return 0
            return self.app.frame.weblistPanel.loadPanel( self )
        return 1

    def copyScreenToTable(self, path ):

        node = self.__getNode( path )

        folder = self.__getFolder( path )

        panels = self.app.frame.weblistPanel.GetChildren()
        fields = self.app.config['fields']
        
        if len(panels) == 0:
            return

        #  copy over the folder details...
        f = panels[0]
        folder['Title'] = f.ttitle.GetValue()
        folder['Description'] = f.tdesc.GetValue()

        #  copy over the photo details...        
        for i in panels[1:]:

            name = i.tname.GetLabel()

            node[name]['Title'] = i.ttitle.GetValue()
            node[name]['Description'] = i.tdesc.GetValue()
            node[name]['Properties'][fields[0]] = i.t1.GetValue()
            node[name]['Properties'][fields[2]] = i.t3.GetValue()            
            node[name]['Properties'][fields[1]] = i.t2.GetValue()

        return 1
    
    def actionString( self ):
        """Puts together the common part of the action String"""
        return 'self.xmlrpc.' + string.join(self.getCurrentPath(),'.')

    def saveImage( self, id, filename ):
        """ saves the image up to the current folder """
        image = Image.open(filename)
        if not image:
            return 0
            
        image = image.resize((300,300))
        image.save("c:\\picture\\temp.jpg")

        imgdata = open("c:\\picture\\temp.jpg",'rb').read()
        self.imagedata = xmlrpclib.Binary(imgdata)

        return self.__perform( self.actionString() + '.' + id +  ".PhotoData(self.imagedata)" )

    def addImageData(self, filename, topathstr, id):            

        image = wx.Image(filename)
        width = image.GetWidth()
        height = image.GetHeight()
        
        insize = width
        if height > width:
            insize = height

        ratio = float(self.app.isize) / insize
        mywidth = ratio * width
        myheight = ratio * height
        print 'isize ' + str(self.app.isize) + ' width' + str(width) + ' height ' + str(height) + ' ratio ' + str(ratio) + 'mywidth ' + str(mywidth) + ' myheight ' + str(myheight)
        image = image.Scale(mywidth,myheight)
        stagefile = self.app.cachedir + 'stage.jpg'
        
        image.SaveFile( stagefile, wx.BITMAP_TYPE_JPEG )
        print 'addimagedata'
        imgdata = open(stagefile,'rb').read()
        self.data = xmlrpclib.Binary(imgdata)

        return self.__perform( 'self.xmlrpc.' + topathstr + '.' + id +  ".PhotoData(self.data)")


    def addPhoto( self, targetItem, filename ):
        
        topathstr    = string.join( self.__getPath( targetItem ), '.')

        path,name = os.path.split(filename)
        id = cleanup( name.replace('.jpg','') )
        
        action = 'self.xmlrpc.' + topathstr + '.xmlrpc_create(\'Photo\',\'' + id + '\')' 
        print 'movedataoption is ' +  str( self.app.config['MoveDataOption'] )

        if not self.__perform( action ):
            return 0

        ans = 0            
        if self.app.config['MoveDataOption'] == 2:
            dlg = wx.MessageDialog(self.app.frame, 'Move image data now? (this can take time)', 'Upload image data', wx.YES_NO )
            ans = dlg.ShowModal()
            dlg.Destroy()

        if self.app.config['MoveDataOption']== 0 or ans == 5103:
            self.addImageData( filename, topathstr, id)
        else:
            self.movedatalater.append([filename,topathstr,id])

        image = wx.Image(filename)
        width = image.GetWidth()
        height = image.GetHeight()
        
        if width > height:
            image.Rescale( 120, 90 )
        else:
            image.Rescale( 90, 120 )

        #  now pop it in the cache        

        path = string.join( self.__getPath( targetItem ), '~' )
        outfilename = self.app.cachedir + path + '~' + id + '.jpg' 

        image.SaveFile(outfilename, wx.BITMAP_TYPE_JPEG)
        imgdata = open(outfilename,'rb').read()
        self.thumbnail = xmlrpclib.Binary(imgdata)

        fields = self.app.config['fields']
        #  create a tree node...

        here = self.__getNode( self.__getPath( targetItem ) )
        here[id] = {'Type':'Photo',
                       'Title':'',
                       'Description':'',
                       'Properties':{fields[0]:'', fields[1]:'', fields[2]:''},
                       'filename':'',
                       'data':self.thumbnail};

        #  and reload the panel
        
        self.app.frame.weblistPanel.loadPanel( self )
        return 1
        #  now show the weblist panel

    def MakePDF( self ):
        self.__perform( self.actionString()  + '.makePDF()')
        win32api.ShellExecute(0,None,'c://aardvark//clients//galleries//basedoc.pdf',None,'c://Windows//System32//',1)        

    def addPhotoAlbum( self, id ):
        id = cleanup(id)

        if not self.__perform( self.actionString() + '.xmlrpc_create(\'Photo Album\',\'' + id + '\')' ):
            return 0

        # add the new folder to the tree

        node   = self.getCurrentNode()
        node[id]  = {'Type':'PhotoAlbum',
                      'Title':id,
                      'Description':'',
                      'Node':{} }

        # add the item to the treectrl
        
        child = self.AppendItem(self.GetSelection(), id)
        self.SetItemImage(child, self.fldridx, wx.TreeItemIcon_Normal)
        self.SetItemImage(child, self.fldropenidx, wx.TreeItemIcon_Expanded)
        self.SetItemText(child, id)
        self.Expand(self.GetSelection() )
        self.SelectItem(child)
        self.Refresh()
        return 1
        
    def setdescription(self, id, description):
        return self.__perform( self.actionString()  + '.' + id +  ".setdescription('" + description + "')" )
        
    def setTitle(self, id, title):
        return self.__perform( self.actionString() + '.' + id +  ".setTitle('" + title + "')" )

    def deletePhoto( self, id ):
        dlg = wx.MessageDialog(self.app.frame, 'Delete this Photo?','Confirm Delete', wx.YES_NO | wx.ICON_QUESTION)

        if dlg.ShowModal() == 5103:
            if not self.__perform( self.actionString()  + '.xmlrpc_delete(\'' + id +  '\')' ):
                return 0

            # Alter the tree

            node   = self.getCurrentNode()
            del node[id]
        dlg.Destroy()        
        return 1

    def deleteFolder( self ):
        dlg = wx.MessageDialog(self.app.frame, 'Delete this folder?','Confirm Delete', wx.YES_NO | wx.ICON_QUESTION)

        if dlg.ShowModal() == 5103:

            path = self.getCurrentPath()
            action = 'self.xmlrpc.' + string.join( path[0:-1] , '.' ) + '.xmlrpc_delete(\'' + path[-1] +  '\')' 
            if not self.__perform( action ):
                return 0
            node   = self.__getNode(path[0:-1])
            del node[path[-1]]

            self.deleting = True
            this = self.GetSelection()
            self.SelectItem( self.GetItemParent( this ))
            self.Delete( this )
            self.deleting = False

        dlg.Destroy()        
        return 1

    def __moveObject( self, targetItem, path, id ):
        """Handles the movement of objects: inside Zope, in the image cache and on the tree."""
        frompathstr  = string.join( path, '.')
        topathstr    = string.join( self.__getPath( targetItem ), '.')

        # Do the move inside Zope
        
        action = 'self.xmlrpc.' + topathstr + '.xmlrpc_paste(self.xmlrpc.' + frompathstr + '.xmlrpc_cut(\'' + id + '\'))'
        if not self.__perform(action):
            return 0

        # Alter the tree

        tonode   = self.__getNode( self.__getPath( targetItem ) )
        fromnode = self.__getNode( path )

        tonode[id] = fromnode[id]
        del fromnode[id]

        # Now adjust the thumbnail cache to reflect the move

        cachedir = self.app.cachedir
        oldpath  = frompathstr.replace('.','~') + '~' + id
        newpath  = topathstr.replace('.','~') + '~' + id

        i = len( oldpath )

        for imagename in os.listdir( cachedir ):

            if imagename[0:i] == oldpath:

                newname = imagename.replace( oldpath, newpath )
                if os.access(cachedir + newname, os.F_OK ):
                    os.remove( cachedir + newname )
                os.rename( cachedir + imagename, cachedir + newname ) 
                wx.LogMessage('os.rename ' + imagename + ' ' + newname)

        return 1
    
    def moveFolder( self, targetItem ):
        """ targetItem    - a treeCtrl node """
        
        if len( self.movedatalater ) > 0:
            dlg = wx.MessageDialog(self.app.frame, "You must upload new photos before you can move things around. This will take some time to complete. Upload Now?", 'Save Images Now?', wx.YES_NO )
            if dlg.ShowModal() == 5103:   # YES
                self.saveToWeb()
            else:
                return 0

        path     = self.getCurrentPath()
        id       = path.pop()

        dest = self.__getPath( targetItem )

        while len( dest ) > len( path ):
            dest = dest[:-1]
            
        if path == dest:
            return 0

        oldItem = self.GetSelection()
        newItem = self.AppendItem( targetItem, self.GetItemText( oldItem ), data=self.GetItemData( oldItem ) )

        self.SetItemImage(newItem, self.fldridx, wx.TreeItemIcon_Normal)
        self.SetItemImage(newItem, self.fldropenidx, wx.TreeItemIcon_Expanded)

        self.deleting = True
        self.SelectItem( newItem )

        if not self.__moveObject( targetItem, path, id ):
            return 0
   
        self.Delete( oldItem )
        self.deleting = False
        return 1
    
    def movePhoto( self, targetItem, id ):
        """ targetItem    - a treeCtrl node
            source        - id of image to move """
        
        if len( self.movedatalater ) > 0:
            dlg = wx.MessageDialog(self.app.frame, "You must upload new photos before you can move things around. This will take some time to complete. Upload Now?", 'Save Images Now?', wx.YES_NO )
            if dlg.ShowModal() == 5103:   # YES
                self.saveToWeb()
            else:
                return 0        
        if self.getCurrentPath() == self.__getPath( targetItem ):
            return 0
        return self.__moveObject( targetItem, self.getCurrentPath(), id )
        
    def saveToWeb(self):

        while len(self.dirty) > 0:
            item = self.dirty.pop()
            path = item['path'].split('~')

            if item['item'] == 0:
                pathstr = string.join(path,'.')
                here = self.__getFolder(path)
            else:
                path = prepend( path, item['item'] )
                pathstr = string.join(path,'.')
                here = self.__getNode(path)

            if item['field'] in self.app.propitems:
                here = here['Properties']
                action = 'self.xmlrpc.' + pathstr + '.set_property("' + item['field'] + '","' + esc(here[item['field']]) + '")'
            else:
                action = 'self.xmlrpc.' + pathstr + ".set" + item['field'] + '("' + esc(here[item['field']]) + '")'

            if not self.__perform(action):
                return 0

        for pic in self.movedatalater:
            msg = wx.MessageDialog(self, 'Copying ' + pic[0] + ' to website','Copying Image Data',style=wx.ICON_INFORMATION)
            msg.Show()
            self.addImageData( pic[0], pic[1], pic[2] )
            msg.Destroy()            
        self.movedatalater = []
        return 1
        
class MyFrame( wx.Frame ):
    """GUI component of the gallery program. Works in conjunction with
       MyTree, which contains the xmlrpc logic and user data """
    
    def MessageDialog(self, text, title):
        messageDialog = wx.MessageDialog(self, text, title, wx.OK | wx.ICON_INFORMATION)
        messageDialog.ShowModal()
        messageDialog.Destroy()

    def __init__(self, app, title):
        wx.Frame.__init__(self, None, -1, title)
        self.app = app
        self.SetBackgroundColour( app.c1 )

        #  set up the local files folder

        if not self.app.config.has_key('cameradir'):
            if not self.ChangeLocalFolder():
                return
        # now show the splash screen
        self.app.splash.Show()        
        #  Make the panels

        splitter            = wx.SplitterWindow( self,              -1, style = wx.NO_BORDER)
        logsplitter         = wx.SplitterWindow( splitter,          -1, style = wx.NO_BORDER)

        splitter.SetBackgroundColour(app.c1)
        logsplitter.SetBackgroundColour(app.c1)

        self.leftPanel      = wx.Window(         splitter,          -1  , style = wx.NO_BORDER  )
        self.leftPanel.SetBackgroundColour(app.c1)

        self.notebookPanel = wx.Notebook( logsplitter, -1, style = wx.NO_BORDER | wx.NB_TOP )
        self.notebookPanel.SetBackgroundColour( app.c1 )
        self.log = wx.TextCtrl(logsplitter, -1, style = wx.TE_MULTILINE|wx.TE_READONLY|wx.HSCROLL| wx.NO_BORDER)
        self.log.SetBackgroundColour( app.c1 )
        self.log.SetForegroundColour( "Grey" )

        self.weblistPanel = TestPanel( self.notebookPanel, app )
        self.weblistPanel.SetBackgroundColour( app.c1 )
        self.photolistPanel = CameraPanel( self.notebookPanel, app )
        self.photolistPanel.SetBackgroundColour( app.c1 )
        
        self.notebookPanel.AddPage(self.weblistPanel, 'Web Site')
        self.notebookPanel.AddPage(self.photolistPanel, 'Local Files')
        wx.Log_SetActiveTarget(MyLog(app, self.log))

        logsplitter.SetMinimumPaneSize(20)
        logsplitter.SplitHorizontally(self.notebookPanel,self.log, 200)
        
        splitter.SetMinimumPaneSize(20)
        splitter.SplitVertically(self.leftPanel, logsplitter, 200)
        logsplitter.Bind(wx.EVT_SIZE, self.SplitterOnSize)

        #  Make the menus

        menuBar = wx.MenuBar()
        menuBar.SetBackgroundColour( app.c1 )
        menu = wx.Menu()
        options = wx.Menu()
        edit = wx.Menu()
        helpmenu = wx.Menu()

        menuBar.Append(menu, "&File")
        menuBar.Append(edit, "&Edit")
        menuBar.Append(options, "&Options")
        menuBar.Append(helpmenu, "&Help")

        menu.Append(     wx.ID_SAVE, "S&ave\tCtrl-S", "Save your changes to the website" )
        menu.Append(     92 , "&New Folder\tCtrl-F" , "Add a new Folder"                 )
        menu.Append(     95 , "New &Root Folder\tCtrl-R" , "Add a new Root Folder"                 )                
##        menu.Append(     93 , "Make &PDF\tCtrl-P" , "Create a PDF of this Folder"        )        
        menu.Append(     wx.ID_EXIT, "E&xit\tAlt-F4", "Exit this program"                )

        options.Append( 304, "Change local images folder", "Change the default PC folder for images." )
        options.Append( 302, "Ask each time", "Prompt for move now / move later option whenever I upload a picture", wx.ITEM_RADIO )
        options.Append( 300, "Move Data Now", "Copy new images to the site as soon as I add them (can cause delays).", wx.ITEM_RADIO )
        options.Append( 301, "Move Data Later", "Don't copy the new images up until I've finished working on my site.", wx.ITEM_RADIO )


        edit.Append( 401, "Cut\tCTRL-X", "Cut" )
        edit.Append( 402, "Copy\tCTRL-C", "Copy" )
        edit.Append( 403, "Paste\tCTRL-V", "Paste" )        
        edit.Append( 404, "Delete\tDEL", "Del" )        
        
        helpmenu.Append( wx.ID_HELP, "&About\tAlt-A", "About the program"                )
        
        self.Bind( wx.EVT_MENU, self.OnSave,        id=wx.ID_SAVE )
        self.Bind( wx.EVT_MENU, self.OnAddFolder,    id=92        )
        self.Bind( wx.EVT_MENU, self.OnAddRootFolder,    id=95        )
        self.Bind( wx.EVT_MENU, self.OnMakePDF,    id=93        )
        self.Bind( wx.EVT_CLOSE, self.OnTimeToClose               )
        self.Bind( wx.EVT_MENU, self.OnTimeToClose, id=wx.ID_EXIT )
        self.Bind( wx.EVT_TOOL, self.OnSave,        id=20         )
        self.Bind( wx.EVT_MENU, self.OnHelpAbout,   id=wx.ID_HELP )      

        self.Bind(wx.EVT_MENU, self.OnChangeLocalFolder, id=304)
        
        self.Bind(wx.EVT_MENU, self.Menu301To303, id=300)
        self.Bind(wx.EVT_MENU, self.Menu301To303, id=301)
        self.Bind(wx.EVT_MENU, self.Menu301To303, id=302)

        self.Bind(wx.EVT_MENU, self.docut, id=401)
        self.Bind(wx.EVT_MENU, self.docopy, id=402)
        self.Bind(wx.EVT_MENU, self.dopaste, id=403)
        self.Bind(wx.EVT_MENU, self.dodelete, id=404)
        
        self.SetMenuBar(menuBar)

        # Make the toolbar
        
        tb = self.CreateToolBar( wx.TB_HORIZONTAL | wx.NO_BORDER | wx.TB_FLAT | wx.TB_TEXT )
        tb.SetBackgroundColour( app.c1 )
        tb.SetToolBitmapSize(wx.Size(24,24))
       
        saveicon = wx.Image(opj('bitmaps\save3.jpg'), wx.BITMAP_TYPE_JPEG).ConvertToBitmap()
        tb.AddSimpleTool(20, saveicon, "Save", "Write your changes to the website")
        tb.AddSeparator()
        saveicon = wx.Image(opj('bitmaps\\newf3.gif'), wx.BITMAP_TYPE_GIF).ConvertToBitmap()
        tb.AddSimpleTool(92, saveicon, "New Folder", "Create a new folder")

        tb.Realize()


        return

##HANDLERS

    def docopy(self, evt):
        self.do = wx.TextDataObject()
        self.do.SetText(self.text.GetValue())
        wx.TheClipboard.Open()
        wx.TheClipboard.SetData(self.do)
        wx.TheClipboard.Close()


    def dopaste(self, evt):
        do = wx.TextDataObject()
        wx.TheClipboard.Open()
        success = wx.TheClipboard.GetData(do)
        wx.TheClipboard.Close()
        window = wx.GetActiveWindow()
        window.WriteText( do.GetText() )
    
    def docut(self, evt):
        None

    def dodelete(self,evt):
        self.weblistPanel.Delete()
            
    def SplitterOnSize(self, evt):
        """Make the splitter on the right expand the top window when resized"""                    
        splitter = evt.GetEventObject()
        sz       = splitter.GetSize()
        splitter.SetSashPosition(sz.height - 120, False)
        evt.Skip()

    def ChangeLocalFolder(self):
        dlg = wx.DirDialog(self, "Choose a directory:",
                          style=wx.DD_DEFAULT_STYLE|wx.DD_NEW_DIR_BUTTON)

        if dlg.ShowModal() == wx.ID_OK:
            self.app.config['cameradir'] = dlg.GetPath() + '\\'
            self.app.saveIni()
            return True

        dlg.Destroy()
        return False

    def OnChangeLocalFolder(self, event):
        if self.ChangeLocalFolder():
            self.photolistPanel.loadpanel( self.app.config['cameradir'] )
        
    def Menu301To303(self, event):
        self.app.config["MoveDataOption"] = event.GetId() - 300
        print 'option is '  + str( self.app.config["MoveDataOption"] )
    def OnHelpAbout(self, event):
        self.MessageDialog(self.app.ABOUT_TEXT, "About Widgets Gallery")
        pass

    def OnAddFolder(self,event):
        dlg = wx.TextEntryDialog(self, 'Enter the name of the new folder','Add a new Folder')
    
        if dlg.ShowModal() == wx.ID_OK:
            name = cleanup(dlg.GetValue())

            self.app.tree.addPhotoAlbum( name )

        dlg.Destroy()

    def OnAddRootFolder(self,event):

        self.app.tree.SelectRoot = True
        self.app.tree.SelectItem(self.app.tree.GetRootItem())
        self.app.tree.SelectRoot = False

        dlg = wx.TextEntryDialog(self, 'Enter the name of the new folder','Add a new Folder')
    
        if dlg.ShowModal() == wx.ID_OK:
            name = cleanup(dlg.GetValue())

            self.app.tree.addPhotoAlbum( name )

        dlg.Destroy()


    def OnAddPhoto(self,event):
        dlg = wx.FileDialog(
        self, message="Choose an image", defaultDir="c:\\picture",
        defaultFile="", wildcard="Picture files (*.jpg)|*.jpg", style=wx.OPEN | wx.CHANGE_DIR )

        if dlg.ShowModal() != wx.ID_OK:
            return

        wx.LogMessage(str(dlg.GetStyle()))
        paths = dlg.GetPath()
        filename = dlg.GetFilename()
        parts = string.split(filename, '.')
        PhotoId = cleanup( parts[0] )

        image = Image.open(opj(dlg.GetPath()))
        image = image.resize((300,300))
        image.save("c:\\picture\\temp.jpg")

        app.tree.addPhoto( PhotoId, filename )

    def OnTimeToClose(self, event):
        try:
            self.app.tree.xmlrpc.changeStatus('publish')
        except xmlrpclib.ProtocolError:
            None
        
        if len(self.app.tree.dirty) + len(self.app.tree.movedatalater) == 0:
            self.Destroy()
            return
        
        dlg = wx.MessageDialog(self, 'Save changes to site before quitting?',
                              'Thanks for using Widget Galleries', wx.YES | wx.NO | wx.CANCEL | wx.ICON_QUESTION)

        reply = dlg.ShowModal()

        if reply == 5101:   # CANCEL
            return

        if reply == 5103:   # YES
            if len(self.app.tree.dirty) > 0:
                self.app.tree.copyScreenToTable(app.tree.GetSelection())
                self.app.tree.saveToWeb(path)

            if len(self.app.tree.movedatalater) > 0:
                print 'here'
                for pic in self.app.tree.movedatalater:
                    print 'here2'
                    msg = wx.MessageDialog(self, 'Copying ' + pic[0] + ' to website','Copying Image Data',style=wx.ICON_INFORMATION)
                    msg.Show()
                    self.app.tree.addImageData( pic[0], pic[1], pic[2] )
                    msg.Destroy()
        dlg.Destroy()
        self.Destroy()
        
    def OnSave(self, event):
        app.tree.copyScreenToTable( app.tree.getCurrentPath() )
        app.tree.saveToWeb()
        pickfile = open('products.pik','w')
        pickle.dump(app.tree.images, pickfile)
        pickfile.close()
        return True

    def OnReloadTree(self, event):
        return True

    def OnMakePDF( self, event):
        self.app.tree.MakePDF()

class App( wx.App ):
    def OnInit(self):
        """Set up some global data, initiate the GUI, then set up the data """
        self.c3 = wx.Colour(202,205,153)   ## header 
        self.c1 = wx.Colour(216,219,164)   ## stripe1
        self.c2 = wx.Colour(191,193,145)   ## stripe2        
        self.csel = wx.Colour(140,140,110)     ## selected
        wx.InitAllImageHandlers()

        self.isz = (16,16)
        self.il = wx.ImageList(self.isz[0], self.isz[1])

        self.cachedir  = 'cache\\'
        self.PCcachedir  = 'PCcache\\'        
        self.ABOUT_TEXT = """Plone Widgets Client v0.1 : Copyright 2005 Kieran Martin
        Plone Widgets Client is licensed under the GNU General Public License."""

        self.startup = 1
        self.pix = []
        for file in os.listdir('cache\\'):
            if file[-4:] == '.jpg':
                self.pix.append(file[0:-4])

        self.ABOUT_TEXT = """Widgets Gallery : Copyright 2004 Kieran Martin
        Widgets Gallery was built using Python, wxPython, and wxWindows."""

        config = {}
        config['MoveDataOption'] = 2

        config['fields'] = ['cost','retail','supplier']
        self.propitems = config['fields']
        
##        self.ini = sys.argv[1] + '.ini'
        self.ini = 'WGallery.ini'
        
        y = open(self.ini)
        z = y.readlines()
        y.close()
        config['userid'], url = z[0].split('@')
        config['url'] = url
        if len(z) > 1:
            config['cameradir'] = z[1]

        self.config = config
        
        #  show the splash screen
        splash = MySplashScreen()
        self.splash = splash

        #  open up the GUI components   
        self.frame = MyFrame(self,"Widgets Web Gallery Maintenance Program")

        #  initiate the data
        self.tree = MyTree(self)

        splash.Close()

##        if not self.tree.xmlrpc:
##            self.ExitMainLoop()
##            return False
            
        #    grab the maximum image size defined on the site
        self.isize = 128
        for x in self.tree.xmlrpc.photo_display_sizes().values():
            if x[0] > self.isize:
                self.isize = x[0]
                
        self.frame.Layout()
        self.frame.Maximize()
        self.frame.Show()
        return True

    def saveIni(self):
        y = open(self.ini,'w')
        url = self.config['url'].replace('\n', '')
        y.write(self.config['userid'] + '@' + url + '\n')
        y.write(self.config['cameradir'])
        y.close()
app = App(0)
app.MainLoop()
